@@ -146,7 +146,7 @@ async function render(q) {
     .on('pageerror', error => console.log(`ERROR: ${error.message}`))
     .on('response', response => {
       if (response.status() >= 400)
-        console.log(`HTTP ${response.status()}: ${response.url()}`)
+        console.log(`HTTP ${response.status()}: ${response.request().method()} ${response.url()}`)
     })
     .on('requestfailed', request => console.log(`${request.failure().errorText}: ${request.url()}`))
 
@@ -158,8 +158,12 @@ async function render(q) {
   if (q.cookie) {
     let cookieList = []
     let cookieObj = cookie.parse(q.cookie)
-    for (let key in cookieObj)
-      cookieList.push({name: key, value: cookieObj[key], url: q.url})
+    for (let key in cookieObj) {
+      let cookie = { name: key, value: cookieObj[key], url: q.url }
+      if (q.domain)
+        cookie.domain = q.domain
+      cookieList.push(cookie)
+    }
     await page.setCookie(...cookieList)
     delete headers.cookie
   }
