@@ -13,18 +13,17 @@ import subprocess       # nosec: only enabled via app developer
 import sys
 import tempfile
 import time
+import tornado.ioloop
 import tornado.template
 from threading import Thread
 from queue import Queue
 from orderedattrdict import AttrDict
 from tornado.concurrent import Future
-from tornado.ioloop import IOLoop, PeriodicCallback
 from gramex.config import app_log, merge, used_kwargs, CustomJSONDecoder, CustomJSONEncoder
 from gramex.config import PathConfig
 from urllib.parse import urlparse
 
 
-_undef = object()
 MILLISECOND = 0.001         # in seconds
 _opener_defaults = dict(mode='r', buffering=-1, encoding='utf-8', errors='strict',
                         newline=None, closefd=True)
@@ -131,7 +130,7 @@ def _template(path, **kwargs):
     return tornado.template.Loader(root, **kwargs).load(name)
 
 
-def read_excel(io, sheet_name=0, table=None, name=None, range=None, header=_undef, **kwargs):
+def read_excel(io, sheet_name=0, table=None, name=None, range=None, header=..., **kwargs):
     '''
     Read data from an XLSX as a DataFrame using ``openpyxl``.
 
@@ -150,7 +149,7 @@ def read_excel(io, sheet_name=0, table=None, name=None, range=None, header=_unde
     if not any((range, name, table)):
         # Pandas defaults to xlrd, but we prefer openpyxl
         kwargs.setdefault('engine', 'openpyxl')
-        return pd.read_excel(io, sheet_name=sheet_name, header=0 if header is _undef else header,
+        return pd.read_excel(io, sheet_name=sheet_name, header=0 if header is ... else header,
                              **kwargs)
 
     import openpyxl
@@ -161,7 +160,7 @@ def read_excel(io, sheet_name=0, table=None, name=None, range=None, header=_unde
     if table is not None:
         range = ws.tables[table].ref
         # Tables themselves specify whether they have a column header. Use this as default
-        if header is _undef:
+        if header is ...:
             header = list(builtin_range(ws.tables[table].headerRowCount))
     elif name is not None:
         # If the name is workbook-scoped, get it directly
@@ -179,7 +178,7 @@ def read_excel(io, sheet_name=0, table=None, name=None, range=None, header=_unde
 
     data = pd.DataFrame([[cell.value for cell in row] for row in ws[range]])
     # Header defaults to 0 if undefined. If it's not None, apply the header
-    header = 0 if header is _undef else header
+    header = 0 if header is ... else header
     if header is not None:
         data = (data.T.set_index(header).T      # Set header rows as column names
                     .reset_index(drop=True)     # Drop index with "holes" where headers were
@@ -876,10 +875,12 @@ def _get_current_ioloop():
     object that mimics add_callback() by running the method immediately.
     This allows daemon() to be run without Tornado / asyncio.
     '''
-    loop = IOLoop.current(instance=False)
-    if loop is None:
-        loop = AttrDict(add_callback=lambda fn, *args, **kwargs: fn(*args, **kwargs))
-    return loop
+    from gramex.services import info
+    return (
+        info.main_ioloop or
+        tornado.ioloop.IOLoop.current() or
+        AttrDict(add_callback=lambda fn, *args, **kwargs: fn(*args, **kwargs))
+    )
 
 
 def get_store(type, **kwargs):
@@ -928,9 +929,9 @@ class KeyStore(object):
                 purge_keys)
         # Periodically flush and purge buffers
         if flush is not None:
-            PeriodicCallback(self.flush, callback_time=flush * 1000).start()
+            tornado.ioloop.PeriodicCallback(self.flush, callback_time=flush * 1000).start()
         if purge is not None:
-            PeriodicCallback(self.purge, callback_time=purge * 1000).start()
+            tornado.ioloop.PeriodicCallback(self.purge, callback_time=purge * 1000).start()
         # Call close() when Python gracefully exits
         atexit.register(self.close)
 
@@ -1215,7 +1216,7 @@ class JSONStore(KeyStore):
 
     def flush(self, purge=False):
         super(JSONStore, self).flush()
-        if self.changed or purge:
+        if getattr(self, 'changed') or purge:
             app_log.debug(f"{'Purging' if purge else 'Flushing'} {self.path}")
             # Don't dump contents. That can overwrite other instances' updates.
             # Instead: read, apply updates, and save.
