@@ -17,7 +17,7 @@ from orderedattrdict import AttrDict
 import gramex
 import gramex.cache
 from gramex.http import UNAUTHORIZED, FORBIDDEN
-from gramex.config import app_log, objectpath, str_utf8, merge
+from gramex.config import app_log, objectpath, merge
 from gramex.transforms import build_transform
 from .basehandler import BaseHandler, build_log_info
 
@@ -51,11 +51,11 @@ class AuthHandler(BaseHandler):
         cls.failed_logins = Counter()
         # Set delay for failed logins from the delay: parameter which can be a number or list
         default_delay = [1, 1, 5]
-        cls.delay = delay
-        if isinstance(cls.delay, list) and not all(isinstance(n, (int, float)) for n in cls.delay):
-            app_log.warning('%s: Ignoring invalid delay: %r', cls.name, cls.delay)
-            cls.delay = default_delay
-        elif isinstance(cls.delay, (int, float)) or cls.delay is None:
+        cls.delay = default_delay if delay is None else delay
+        if not isinstance(cls.delay, list):
+            cls.delay = [cls.delay]
+        if not all(isinstance(n, (int, float)) for n in cls.delay):
+            app_log.warning(f'{cls.name}: Ignoring invalid delay: {cls.delay!r}')
             cls.delay = default_delay
 
         # Set up session user key, session expiry and inactive expiry
@@ -71,7 +71,7 @@ class AuthHandler(BaseHandler):
             if isinstance(lookup, dict):
                 cls.lookup_id = cls.lookup.pop('id', 'user')
             else:
-                app_log.error('%s: lookup must be a dict, not %s', cls.name, cls.lookup)
+                app_log.error(f'{cls.name}: lookup must be a dict, not {cls.lookup}')
 
         cls.rules = gramex.data.filter(**rules) if rules else gramex.data.pd.DataFrame()
         cls.rules.fillna(value='', inplace=True)
@@ -82,14 +82,14 @@ class AuthHandler(BaseHandler):
             cls.auth_methods['prepare'] = build_transform(
                 conf={'function': prepare},
                 vars={'handler': None, 'args': None},
-                filename='url:%s:prepare' % cls.name,
+                filename=f'url:{cls.name}:prepare',
                 iter=False)
         # Prepare recaptcha
         if recaptcha is not None:
             if 'key' not in recaptcha:
-                app_log.error('%s: recaptcha.key missing', cls.name)
+                app_log.error(f'{cls.name}: recaptcha.key missing')
             elif 'key' not in recaptcha:
-                app_log.error('%s: recaptcha.secret missing', cls.name)
+                app_log.error(f'{cls.name}: recaptcha.secret missing')
             else:
                 recaptcha.setdefault('action', 'login')
                 cls.auth_methods['recaptcha'] = cls.check_recaptcha
@@ -102,7 +102,7 @@ class AuthHandler(BaseHandler):
             for conf in action:
                 cls.actions.append(build_transform(
                     conf, vars=AttrDict(handler=None),
-                    filename='url:%s:%s' % (cls.name, conf.function)))
+                    filename=f'url:{cls.name}:{conf.function}'))
 
     @coroutine
     def prepare(self):
@@ -216,7 +216,7 @@ class AuthHandler(BaseHandler):
         response = yield http.fetch(self._RECAPTCHA_VERIFY_URL, method='POST', body=body)
         result = json.loads(response.body)
         if not result['success']:
-            raise HTTPError(FORBIDDEN, 'recaptcha failed: %s' % ', '.join(result['error-codes']))
+            raise HTTPError(FORBIDDEN, f'recaptcha failed: {", ".join(result["error-codes"])}')
 
     def authorize(self):
         '''AuthHandlers don't have authorization. They're meant to log users in.'''
@@ -408,7 +408,7 @@ class OTP(object):
         # create database at GRAMEXDATA
         path = os.path.join(gramex.variables.GRAMEXDATA, 'auth.recover.db')
         url = 'sqlite:///{}'.format(path)
-        self.engine = gramex.data.create_engine(url, encoding=str_utf8)
+        self.engine = gramex.data.create_engine(url, encoding='utf-8')
         conn = self.engine.connect()
         conn.execute('CREATE TABLE IF NOT EXISTS users '
                      '(user TEXT, email TEXT, token TEXT, expire REAL)')
