@@ -9,6 +9,7 @@ import tornado.gen
 import gramex.cache
 from typing import Union
 from binascii import b2a_base64, hexlify
+from fnmatch import fnmatch
 from http.cookies import Morsel
 from orderedattrdict import AttrDict
 from urllib.parse import urlsplit, urlunsplit, parse_qsl, urljoin, urlencode
@@ -24,7 +25,6 @@ from gramex.cache import KeyStore, JSONStore, HDF5Store, SQLiteStore, RedisStore
 
 server_header = f'Gramex/{__version__}'
 session_store_cache = {}
-_arg_default = object()
 
 # Python 3.8+ supports SameSite cookie attribute. Monkey-patch it for Python 3.7
 # https://stackoverflow.com/a/50813092/100904
@@ -33,8 +33,11 @@ Morsel._reserved.setdefault('samesite', 'SameSite')
 
 class BaseMixin(object):
     @classmethod
-    def setup(cls, transform={}, redirect={}, methods=None, auth=None, log=None, set_xsrf=None,
-              error=None, xsrf_cookies=None, **kwargs):
+    def setup(cls, transform={}, redirect={}, methods=None,
+              auth: Union[None, bool, dict] = None,
+              log=None, set_xsrf=None, error=None, xsrf_cookies=None,
+              cors: Union[None, bool, dict] = None,
+              **kwargs):
         '''
         One-time setup for all request handlers. This is called only when
         gramex.yaml is parsed / changed.
@@ -55,6 +58,7 @@ class BaseMixin(object):
         cls.setup_xsrf(xsrf_cookies)
         cls.setup_log()
         cls.setup_httpmethods(methods)
+        cls.setup_cors(cors, auth=auth)
 
         # app.settings.debug enables debugging exceptions using pdb
         if conf.app.settings.get('debug', False):
@@ -62,7 +66,7 @@ class BaseMixin(object):
 
     # A list of special keys for BaseHandler. Can be extended by other classes.
     special_keys = ['transform', 'redirect', 'methods', 'auth', 'log', 'set_xsrf',
-                    'error', 'xsrf_cookies', 'headers']
+                    'error', 'xsrf_cookies', 'cors', 'headers']
 
     @classmethod
     def clear_special_keys(cls, kwargs, *args):
@@ -76,18 +80,143 @@ class BaseMixin(object):
             kwargs.pop(special_key, None)
         return kwargs
 
+    @classmethod
+    def get_list(cls, val: Union[list, tuple, str], key: str = '', eg: str = '', caps=True) -> set:
+        '''
+        Convert val="GET, PUT" into {"GET", "PUT"}.
+        If val is not a string or list/tuple, raise ValueError("url.{key} invalid. e.g. {eg}")
+        '''
+        if isinstance(val, (list, tuple)):
+            val = ' '.join(val)
+        elif not val:
+            val = ''
+        if not isinstance(val, str):
+            err = f'url:{cls.name}.{key}: {val!r} not a string/list'
+            err = err + f', e.g. {eg}' if eg else err
+            raise ValueError(err)
+        if caps:
+            val = val.upper()
+        return set(val.replace(',', ' ').split())
+
     @classmethod
     def setup_httpmethods(cls, methods: Union[list, tuple, str]):
-        if methods is None:
-            return
-        if isinstance(methods, (list, tuple)):
-            methods = ' '.join(methods)
-        if not isinstance(methods, str):
-            raise ValueError(f'methods: {methods!r} invalid -- use a string/list, e.g. [GET, PUT]')
+        methods = cls.get_list(methods, key='methods', eg='[GET, POST]', caps=True)
         if methods:
-            cls._http_methods = set(methods.upper().replace(',', ' ').split())
+            cls._http_methods = methods
             cls._on_init_methods.append(cls.check_http_method)
 
+    def check_http_method(self):
+        '''If method: [...] is specified, reject all methods not in the allowed methods set'''
+        if self.request.method not in self._http_methods:
+            raise HTTPError(METHOD_NOT_ALLOWED, f'{self.name}: method {self.request.method} ' +
+                            f'not in allowed methods {self._http_methods}')
+
+    @classmethod
+    def setup_cors(cls, cors: Union[None, bool, dict], auth):
+        if cors is None:
+            return
+        if cors is True:
+            cors = {
+                'auth': bool(auth),
+                'methods': '*',
+                'headers': '*',
+                'origins': '*',
+            }
+        if not isinstance(cors, dict):
+            app_log.error(f'url:{cls.name}.cors is not a dict/True')
+            return
+        cls._cors = cors
+        # Set default CORS values as a set
+        for key in ('origins', 'methods', 'headers'):
+            cors[key] = cls.get_list(cors.get(key, '*'), f'cors.{key}', '"*"', caps=False)
+        cls._on_init_methods.append(cls.check_cors)
+        cls.options = cls._cors_options
+
+    def check_cors(self):
+        '''
+        For simple CORS requests, send Access-Control-Allow-Origin: <origin>.
+        If request needs credentials, allow it.'''
+        origin, cred = self.cors_origin()
+        if origin:
+            self.set_header('Access-Control-Allow-Origin', origin)
+            if cred:
+                self.set_header('Access-Control-Allow-Credentials', 'true')
+
+    def cors_origin(self):
+        '''
+        Returns the origin to set in Access-Control-Allow-Origin header.
+        '''
+        # If CORS is not enabled, it fails
+        if not self._cors:
+            return None, False
+        # Assume credentials are passed if handler requires Auth or Cookie is passed
+        cred = self._cors['auth'] or self.request.headers.get('Cookie')
+        # If origin: *, then allow all origins
+        origin = self.request.headers.get('Origin', '').lower()
+        if self._cors['origins'] == set('*'):
+            return (origin if cred else '*', cred)
+        # If it matches any of the wildcards, return specific origin
+        for pattern in self._cors['origins']:
+            if fnmatch(origin, pattern.lower()):
+                return origin, cred
+        # If none of the patterns match, it fails
+        return None, cred
+
+    def _cors_options(self, *args, **kwargs):
+        # Check if origin is in cors.origin
+        origin, cred = self.cors_origin()
+        if not origin:
+            origin = self.request.headers.get('Origin', '')
+            raise HTTPError(BAD_REQUEST, f'url:{self.name}: CORS origin {origin} '
+                                         f'not in {self._cors["origins"]}')
+
+        # Check if method is in cors.methods
+        method = self.request.headers.get('Access-Control-Request-Method', '').upper()
+        for pattern in self._cors['methods']:
+            if fnmatch(method, pattern.upper()):
+                break
+        else:
+            raise HTTPError(BAD_REQUEST, f'url:{self.name}: CORS method {method} '
+                                         f'not in {self._cors["methods"]}')
+
+        # Check if headers is in cors.headers
+        headers = self.request.headers.get('Access-Control-Request-Headers', '')
+        headers = self.get_list(headers, 'headers', '', caps=False)
+        allowed_headers = set([h.lower() for h in self._cors['headers']])
+        diff = set()
+        if '*' not in allowed_headers:
+            for header in headers:
+                if header.lower() not in allowed_headers:
+                    diff.add(header)
+        if diff:
+            raise HTTPError(BAD_REQUEST, f'url:{self.name}: CORS headers {diff} '
+                                         f'not in {self._cors["headers"]}')
+
+        # If it succeeds, set relevant headers
+        self.set_header('Access-Control-Allow-Origin', origin)
+        methods = (self._all_methods if '*' in self._cors['methods']
+                   else ', '.join(self._cors['methods']))
+        self.set_header('Access-Control-Allow-Methods', methods)
+        headers |= self._cors['headers']
+        if '*' in headers:
+            headers.remove('*')
+            headers.update(self._all_headers)
+        self.set_header('Access-Control-Allow-Headers', ', '.join(headers))
+        # TODO: Access-Control-Max-Age
+        # TODO: Access-Control-Expose-Headers
+
+    _all_methods = 'GET, HEAD, POST, PUT, DELETE, OPTIONS, PATCH, CONNECT, TRACE'
+    _all_headers = [
+        'Accept',
+        'Cache-Control',
+        'Content-Type',
+        'If-None-Match',
+        'Origin',
+        'Pragma',
+        'Upgrade-Insecure-Requests',
+        'X-Requested-With',
+    ]
+
     @classmethod
     def setup_default_kwargs(cls):
         '''
@@ -240,7 +369,7 @@ class BaseMixin(object):
             cls.redirects = [no_external(method) for method in cls.redirects]
 
     @classmethod
-    def setup_auth(cls, auth):
+    def setup_auth(cls, auth: Union[None, bool, dict]):
         # auth: if there's no auth: in handler, default to app.auth
         if auth is None:
             auth = conf.app.get('auth')
@@ -250,6 +379,8 @@ class BaseMixin(object):
         # Set up the auth
         if isinstance(auth, dict):
             cls._auth = auth
+            cls._auth_methods = cls.get_list(auth.get('methods', ''), 'auth.methods',
+                                             '[GET, POST, OPTIONS]')
             cls._on_init_methods.append(cls.authorize)
             cls.permissions = []
             # Add check for condition
@@ -365,12 +496,6 @@ class BaseMixin(object):
         '''
         cls.check_xsrf_cookie = cls.noop if xsrf_cookies is False else cls.xsrf_ajax
 
-    def check_http_method(self):
-        '''If method: [...] is specified, reject all methods not in the allowed methods set'''
-        if self.request.method not in self._http_methods:
-            raise HTTPError(METHOD_NOT_ALLOWED, f'{self.name}: method {self.request.method} ' +
-                            f'not in allowed methods {self._http_methods}')
-
     def xsrf_ajax(self):
         '''
         TODO: explain things clearly.
@@ -478,6 +603,9 @@ class BaseMixin(object):
         session_id = b2a_base64(os.urandom(24))[:-1]
         kwargs = dict(self._session_cookie)
         kwargs['expires_days'] = expires_days
+        # Treat expiry: false as None, since expiry: null won't set null, it clears YAML
+        if kwargs['expires_days'] is False:
+            kwargs['expires_days'] = None
         # Use Secure cookies on HTTPS to prevent leakage into HTTP
         if self.request.protocol == 'https':
             kwargs['secure'] = True
@@ -486,6 +614,11 @@ class BaseMixin(object):
             self.set_secure_cookie(self._session_cookie_id, session_id, **kwargs)
         except RuntimeError:
             pass
+        # Warn if app.session.domain is x.com but request comes from y.com.
+        host = self.request.host_name
+        if 'domain' in kwargs and not host.endswith(kwargs['domain']):
+            app_log.warning(f'{self.name}: session.domain={kwargs["domain"]} '
+                            f'but cookie sent to {host}')
         return session_id
 
     def get_session(self, expires_days=None, new=False):
@@ -516,6 +649,8 @@ class BaseMixin(object):
         '''
         if expires_days is None:
             expires_days = self._session_expiry
+        # If the expiry time is None, keep in the session store for 1 day
+        store_expires = time.time() + (1 if expires_days is None else expires_days) * 24 * 60 * 60
         created_new_sid = False
         if getattr(self, '_session', None) is None:
             # Populate self._session based on the sid. If there's no sid cookie,
@@ -528,8 +663,7 @@ class BaseMixin(object):
             # Convert bytes session to unicode before using
             session_id = session_id.decode('ascii')
             # If there's no stored session associated with it, create it
-            expires = time.time() + expires_days * 24 * 60 * 60
-            self._session = self._session_store.load(session_id, {'_t': expires})
+            self._session = self._session_store.load(session_id, {'_t': store_expires})
             # Overwrite id to the session ID even if a handler has changed it
             self._session['id'] = session_id
         # At this point, the "sid" cookie and self._session exist and are synced
@@ -542,7 +676,7 @@ class BaseMixin(object):
         if new and not created_new_sid:
             new_sid = self._set_new_session_id(expires_days).decode('ascii')
             # Update expiry and new SID on session
-            s.update(id=new_sid, _t=time.time() + expires_days * 24 * 60 * 60)
+            s.update(id=new_sid, _t=store_expires)
             # Delete old contents. No _t also means expired
             self._session_store.dump(old_sid, {})
 
@@ -553,24 +687,25 @@ class BaseMixin(object):
         if getattr(self, '_session', None) is not None:
             self._session_store.dump(self._session['id'], self._session)
 
-    def otp(self, expire=60):
-        '''
-        Return a one-time password valid for ``expire`` seconds. When the
-        X-Gramex-OTP header
-        '''
-        user = self.current_user
+    def otp(self, expire=60, prefix='otp', user=None):
+        '''Return one-time password valid for ``expire`` seconds. Use with X-Gramex-OTP header'''
+        user = self.current_user if user is None else user
         if not user:
             raise HTTPError(UNAUTHORIZED)
         nbits = 16
         otp = hexlify(os.urandom(nbits)).decode('ascii')
-        self._session_store.dump('otp:' + otp, {'user': user, '_t': time.time() + expire})
+        self._session_store.dump(f'{prefix}:{otp}', {'user': user, '_t': time.time() + expire})
         return otp
 
+    def apikey(self, expire=1e9, user=None):
+        '''Return new API Key. To be used with the X-Gramex-Key header.'''
+        return self.otp(expire=expire, prefix='key', user=user)
+
     def override_user(self):
         '''
         Use ``X-Gramex-User`` HTTP header to override current user for the session.
-        Use ``X-Gramex-OTP`` HTTP header to set user based on OTP.
-        ``?gramex-otp=`` is a synonym for X-Gramex-OTP.
+        Use ``X-Gramex-OTP`` HTTP header to set user based on OTP, or ``?gramex-otp=``.
+        Use ``X-Gramex-Key`` HTTP header to set user based on API key, or ``?gramex-key=``.
         '''
         headers = self.request.headers
         cipher = headers.get('X-Gramex-User')
@@ -586,16 +721,23 @@ class BaseMixin(object):
                 app_log.debug(f'{self.name}: Overriding user to {user!r}')
                 self.session['user'] = user
                 return
+        # If valid OTP is specified, set the user from the API key
         otp = headers.get('X-Gramex-OTP') or self.get_argument('gramex-otp', None)
         if otp:
             otp_data = self._session_store.load('otp:' + otp, None)
             if not isinstance(otp_data, dict) or '_t' not in otp_data or 'user' not in otp_data:
-                raise HTTPError(BAD_REQUEST, f'{self.name}: invalid X-Gramex-OTP: {otp}')
+                raise HTTPError(BAD_REQUEST, f'{self.name}: invalid Gramex OTP: {otp}')
             elif otp_data['_t'] < time.time():
-                raise HTTPError(BAD_REQUEST, f'{self.name}: expired X-Gramex-OTP: {otp}')
+                raise HTTPError(BAD_REQUEST, f'{self.name}: expired Gramex OTP: {otp}')
             self._session_store.dump(f'otp:{otp}', None)
             self.session['user'] = otp_data['user']
-        # ToDo: apikey = headers.get('X-Gramex-OTP') or self.get_argument('gramex-otp', None)
+        # If API Key is specified, set the user from the API key
+        key = headers.get('X-Gramex-Key') or self.get_argument('gramex-key', None)
+        if key:
+            key_data = self._session_store.load('key:' + key, None)
+            if not isinstance(key_data, dict) or 'user' not in key_data:
+                raise HTTPError(BAD_REQUEST, f'{self.name}: invalid Gramex Key: {key}')
+            self.session['user'] = key_data['user']
 
     def set_last_visited(self):
         '''
@@ -645,7 +787,7 @@ class BaseHandler(RequestHandler, BaseMixin):
         elif 'X-HTTP-Method-Override' in self.request.headers:
             self.request.method = self.request.headers['X-HTTP-Method-Override'].upper()
 
-    def get_arg(self, name, default=_arg_default, first=False):
+    def get_arg(self, name, default=..., first=False):
         '''
         Returns the value of the argument with the given name. Similar to
         ``.get_argument`` but uses ``self.args`` instead.
@@ -659,7 +801,7 @@ class BaseHandler(RequestHandler, BaseMixin):
         ``self.args`` is always UTF-8 decoded unicode. Whitespaces are stripped.
         '''
         if name not in self.args:
-            if default is _arg_default:
+            if default is ...:
                 raise MissingArgumentError(name)
             return default
         return self.args[name][0 if first else -1]
@@ -705,6 +847,13 @@ class BaseHandler(RequestHandler, BaseMixin):
         self._exception = traceback.format_exception_only(typ, value)[0].strip()
 
     def authorize(self):
+        # If specific methods are mentioned, authorize only if a mentioned method is used
+        auth_methods = getattr(self, '_auth_methods', None)
+        if auth_methods and self.request.method not in auth_methods:
+            return
+        # If CORS auth is specified, don't authorize for OPTIONS (pre-flight request)
+        if self.request.method == 'OPTIONS' and getattr(self, '_cors', {}).get('auth'):
+            return
         if not self.current_user:
             # Redirect non-AJAX requests GET/HEAD to login URL (if it's a string)
             ajax = self.request.headers.get('X-Requested-With', '').lower() == 'xmlhttprequest'
