@@ -65,7 +65,7 @@ class FormHandler(BaseHandler):
                 cls.modify_all = staticmethod(build_transform(
                     conf={'function': conf_kwargs.pop('modify', None)},
                     vars=cls.function_vars['modify'],
-                    filename='%s.%s' % (cls.name, 'modify'), iter=False))
+                    filename=f'{cls.name}.modify', iter=False))
             cls.datasets = conf_kwargs
             cls.single = False
         # Apply defaults to each key
@@ -76,10 +76,10 @@ class FormHandler(BaseHandler):
         # Ensure that each dataset is a dict with a url: key at least
         for key, dataset in list(cls.datasets.items()):
             if not isinstance(dataset, dict):
-                app_log.error('%s: %s: must be a dict, not %r' % (cls.name, key, dataset))
+                app_log.error(f'{cls.name}: {key}: must be a dict, not {dataset!r}')
                 del cls.datasets[key]
             elif 'url' not in dataset:
-                app_log.error('%s: %s: does not have a url: key' % (cls.name, key))
+                app_log.error(f'{cls.name}: {key}: does not have a url: key')
                 del cls.datasets[key]
             # Ensure that id: is a list -- if it exists
             if 'id' in dataset and not isinstance(dataset['id'], list):
@@ -91,7 +91,7 @@ class FormHandler(BaseHandler):
                 'kwargs': dataset.pop('kwargs', None)
             }
             if conf['function'] is not None:
-                fn_name = '%s.%s.transform' % (cls.name, key)
+                fn_name = f'{cls.name}.{key}.transform'
                 dataset['transform'] = build_transform(
                     conf, vars={'data': None, 'handler': None}, filename=fn_name, iter=False)
             # Convert modify: and prepare: into a data = modify(data) function
@@ -100,7 +100,7 @@ class FormHandler(BaseHandler):
                     dataset[fn] = build_transform(
                         conf={'function': dataset[fn]},
                         vars=fn_vars,
-                        filename='%s.%s.%s' % (cls.name, key, fn), iter=False)
+                        filename=f'{cls.name}.{key}.{fn}', iter=False)
 
     def _options(self, dataset, args, path_args, path_kwargs, key):
         """For each dataset, prepare the arguments."""
@@ -120,7 +120,7 @@ class FormHandler(BaseHandler):
             for k, v in filter_kwargs.pop('default', {}).items()
         }
         # /(.*)/(.*) become 2 path arguments _0 and _1
-        defaults.update({'_%d' % k: [v] for k, v in enumerate(path_args)})
+        defaults.update({f'_{k}': [v] for k, v in enumerate(path_args)})
         # /(?P<x>\d+)/(?P<y>\d+) become 2 keyword arguments x and y
         defaults.update({k: [v] for k, v in path_kwargs.items()})
         args = merge(namespaced_args(args, key), defaults, mode='setdefault')
@@ -157,11 +157,11 @@ class FormHandler(BaseHandler):
             try:
                 result[key] = yield val
             except ValueError as e:
-                app_log.exception('%s: filter failed' % self.name)
-                raise HTTPError(BAD_REQUEST, reason=e.args[0])
+                app_log.exception(f'{self.name}: filter failed')
+                raise HTTPError(BAD_REQUEST, e.args[0])
             except Exception as e:
-                app_log.exception('%s: filter failed' % self.name)
-                raise HTTPError(INTERNAL_SERVER_ERROR, reason=repr(e))
+                app_log.exception(f'{self.name}: filter failed')
+                raise HTTPError(INTERNAL_SERVER_ERROR, repr(e))
             modify = self.datasets[key].get('modify', None)
             if callable(modify):
                 result[key] = modify(data=result[key], key=key, handler=self)
@@ -180,7 +180,7 @@ class FormHandler(BaseHandler):
             elif isinstance(val, bytes):
                 format_options[key] = val.decode('utf-8').format(**params)
         if opt.download:
-            self.set_header('Content-Disposition', 'attachment;filename=%s' % opt.download)
+            self.set_header('Content-Disposition', f'attachment;filename={opt.download}')
         if opt.meta_header:
             self.set_meta_headers(meta)
         result = result['data'] if self.single else result
@@ -200,12 +200,12 @@ class FormHandler(BaseHandler):
             meta[key] = AttrDict()
             opt = self._options(dataset, self.args, path_args, path_kwargs, key)
             if 'id' not in opt.filter_kwargs:
-                raise HTTPError(BAD_REQUEST, reason='%s: need id: kwarg to %s' % (
-                    self.name, self.request.method))
+                raise HTTPError(
+                    BAD_REQUEST, f'{self.name}: need id: in kwargs: to {self.request.method}')
             missing_args = [col for col in opt.filter_kwargs['id'] if col not in opt.args]
             if method != gramex.data.insert and len(missing_args) > 0:
-                raise HTTPError(BAD_REQUEST, reason='%s: missing column(s) in URL query: %s' % (
-                    self.name, ', '.join(missing_args)))
+                raise HTTPError(BAD_REQUEST, f'{self.name}: missing column(s) in URL query: ' +
+                                ', '.join(missing_args))
             # Execute the query. This returns the count of records updated
             result[key] = method(meta=meta[key], args=opt.args, **opt.filter_kwargs)
             # method() should set the schema only on first load. Pop it once done
@@ -214,7 +214,7 @@ class FormHandler(BaseHandler):
             modify = self.datasets[key].get('modify', None)
             if callable(modify):
                 meta[key]['modify'] = modify(data=result[key], key=key, handler=self)
-            self.set_header('Count-%s' % key, val)
+            self.set_header(f'Count-{key}', val)
         # modify the result for multiple datasets
         if hasattr(self, 'modify_all'):
             meta['modify'] = self.modify_all(data=result, key=None, handler=self)
@@ -245,7 +245,7 @@ class FormHandler(BaseHandler):
         if fmt in self.formats:
             fmt = dict(self.formats[fmt])
         else:
-            app_log.error('%s: _format=%s unknown. Using _format=json' % (self.name, fmt))
+            app_log.error(f'{self.name}: _format={fmt} unknown. Using _format=json')
             fmt = dict(self.formats['json'])
 
         # Set up default headers, and over-ride with headers for the format
