@@ -1,6 +1,5 @@
 '''Python-PPTX customized module.'''
 import os
-import six
 import copy
 import logging
 import requests
@@ -39,7 +38,7 @@ def template(tmpl, data):
 def text(shape, spec, data):
     '''Replace entire text of shape with spec['text']'''
     if not shape.has_text_frame:
-        logging.error('"%s" is not a TextShape to apply text:', shape.name)
+        logging.error(f'"{shape.name}" is not a TextShape to apply text:')
         return
     if not isinstance(data, (dict,)):
         data = {'data': data}
@@ -103,7 +102,7 @@ def text(shape, spec, data):
 def replace(shape, spec, data):
     '''Replace keywords in shape using the dictionary at spec['replace']'''
     if not shape.has_text_frame:
-        logging.error('"%s" is not a TextShape to apply text:', shape.name)
+        logging.error(f'"{shape.name}" is not a TextShape to apply text:')
         return
     if not isinstance(data, (dict,)):
         data = {'data': data}
@@ -949,7 +948,7 @@ def css(shape, spec, data):
         if setprop:
             if not isinstance(style[prop], (dict,)):
                 style[prop] = {'function': '{}'.format(style[prop]) if not isinstance(
-                               style[prop], (str, six.string_types,)) else style[prop]}
+                    style[prop], str) else style[prop]}
             setprop = compile_function(style, prop, data, handler)
             setprop = setprop * pxl_to_inch
         else:
